@@ -49,167 +49,49 @@
 #include "guestfs.h"
 #include "guestfs-utils.h"
 
-void
-guestfs_int_free_string_list (char **argv)
+/**
+ * Replace every instance of C<s1> appearing in C<str> with C<s2>.  A
+ * newly allocated string is returned which must be freed by the
+ * caller.  If allocation fails this can return C<NULL>.
+ *
+ * For example:
+ *
+ *  replace_string ("abcabb", "ab", "a");
+ *
+ * would return C<"acab">.
+ */
+char *
+guestfs_int_replace_string (const char *str, const char *s1, const char *s2)
 {
-  size_t i;
+  const size_t len = strlen (str), s1len = strlen (s1), s2len = strlen (s2);
+  size_t i, n;
+  char *ret;
 
-  if (argv == NULL)
-    return;
+  /* Count the size of the final string. */
+  n = 0;
+  for (i = 0; i < len; ++i) {
+    if (strncmp (&str[i], s1, s1len) == 0)
+      n += s2len;
+    else
+      n++;
+  }
 
-  for (i = 0; argv[i] != NULL; ++i)
-    free (argv[i]);
-  free (argv);
-}
-
-size_t
-guestfs_int_count_strings (char *const *argv)
-{
-  size_t r;
-
-  for (r = 0; argv[r]; ++r)
-    ;
-
-  return r;
-}
-
-char **
-guestfs_int_copy_string_list (char *const *argv)
-{
-  const size_t n = guestfs_int_count_strings (argv);
-  size_t i, j;
-  char **ret;
-
-  ret = malloc ((n+1) * sizeof (char *));
+  ret = malloc (n+1);
   if (ret == NULL)
     return NULL;
-  ret[n] = NULL;
 
-  for (i = 0; i < n; ++i) {
-    ret[i] = strdup (argv[i]);
-    if (ret[i] == NULL) {
-      for (j = 0; j < i; ++j)
-        free (ret[j]);
-      free (ret);
-      return NULL;
+  n = 0;
+  for (i = 0; i < len; ++i) {
+    if (strncmp (&str[i], s1, s1len) == 0) {
+      strcpy (&ret[n], s2);
+      n += s2len;
     }
-  }
-
-  return ret;
-}
-
-/* Note that near-identical functions exist in the daemon. */
-char *
-guestfs_int_concat_strings (char *const *argv)
-{
-  return guestfs_int_join_strings ("", argv);
-}
-
-char *
-guestfs_int_join_strings (const char *sep, char *const *argv)
-{
-  size_t i, len, seplen, rlen;
-  char *r;
-
-  seplen = strlen (sep);
-
-  len = 0;
-  for (i = 0; argv[i] != NULL; ++i) {
-    if (i > 0)
-      len += seplen;
-    len += strlen (argv[i]);
-  }
-  len++; /* for final \0 */
-
-  r = malloc (len);
-  if (r == NULL)
-    return NULL;
-
-  rlen = 0;
-  for (i = 0; argv[i] != NULL; ++i) {
-    if (i > 0) {
-      memcpy (&r[rlen], sep, seplen);
-      rlen += seplen;
-    }
-    len = strlen (argv[i]);
-    memcpy (&r[rlen], argv[i], len);
-    rlen += len;
-  }
-  r[rlen] = '\0';
-
-  return r;
-}
-
-/**
- * Split string at separator character C<sep>, returning the list of
- * strings.  Returns C<NULL> on memory allocation failure.
- *
- * Note (assuming C<sep> is C<:>):
- *
- * =over 4
- *
- * =item C<str == NULL>
- *
- * aborts
- *
- * =item C<str == "">
- *
- * returns C<[]>
- *
- * =item C<str == "abc">
- *
- * returns C<["abc"]>
- *
- * =item C<str == ":">
- *
- * returns C<["", ""]>
- *
- * =back
- */
-char **
-guestfs_int_split_string (char sep, const char *str)
-{
-  size_t i, n, c;
-  const size_t len = strlen (str);
-  char reject[2] = { sep, '\0' };
-  char **ret;
-
-  /* We have to handle the empty string case differently else the code
-   * below will return [""].
-   */
-  if (str[0] == '\0') {
-    ret = malloc (1 * sizeof (char *));
-    if (!ret)
-      return NULL;
-    ret[0] = NULL;
-    return ret;
-  }
-
-  for (n = i = 0; i < len; ++i)
-    if (str[i] == sep)
+    else {
+      ret[n] = str[i];
       n++;
-
-  /* We always return a list of length 1 + (# separator characters).
-   * We also have to add a trailing NULL.
-   */
-  ret = malloc ((n+2) * sizeof (char *));
-  if (!ret)
-    return NULL;
-  ret[n+1] = NULL;
-
-  for (n = i = 0; i <= len; ++i, ++n) {
-    c = strcspn (&str[i], reject);
-    ret[n] = strndup (&str[i], c);
-    if (ret[n] == NULL) {
-      for (i = 0; i < n; ++i)
-        free (ret[i]);
-      free (ret);
-      return NULL;
     }
-    i += c;
-    if (str[i] == '\0') /* end of string? */
-      break;
   }
+  ret[n] = '\0';
 
   return ret;
 }
