@@ -1,5 +1,5 @@
 /* libguestfs - the guestfsd daemon
- * Copyright (C) 2009-2016 Red Hat Inc.
+ * Copyright (C) 2009-2020 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,12 +32,12 @@
 #include "actions.h"
 #include "optgroups.h"
 
-GUESTFSD_EXT_CMD(str_lvm, lvm);
+#define MAX_ARGS 64
 
 int
 optgroup_lvm2_available (void)
 {
-  return prog_exists (str_lvm);
+  return prog_exists ("lvm");
 }
 
 /* LVM actions.  Keep an eye on liblvm, although at the time
@@ -103,89 +103,6 @@ convert_lvm_output (char *out, const char *prefix)
   return take_stringsbuf (&ret);
 }
 
-/* Filter a colon-separated output of
- *   lvs -o lv_attr,vg_name,lv_name
- * removing thin layouts, and building the device path as we expect it.
- *
- * This is used only when lvm has no -S.
- */
-static char **
-filter_convert_old_lvs_output (char *out)
-{
-  char *p, *pend;
-  CLEANUP_FREE_STRINGSBUF DECLARE_STRINGSBUF (ret);
-
-  p = out;
-  while (p) {
-    size_t len;
-    char *saveptr;
-    char *lv_attr, *vg_name, *lv_name;
-
-    pend = strchr (p, '\n');	/* Get the next line of output. */
-    if (pend) {
-      *pend = '\0';
-      pend++;
-    }
-
-    while (*p && c_isspace (*p))	/* Skip any leading whitespace. */
-      p++;
-
-    /* Sigh, skip trailing whitespace too.  "pvs", I'm looking at you. */
-    len = strlen (p)-1;
-    while (*p && c_isspace (p[len]))
-      p[len--] = '\0';
-
-    if (!*p) {			/* Empty line?  Skip it. */
-    skip_line:
-      p = pend;
-      continue;
-    }
-
-    lv_attr = strtok_r (p, ":", &saveptr);
-    if (!lv_attr)
-      goto skip_line;
-
-    vg_name = strtok_r (NULL, ":", &saveptr);
-    if (!vg_name)
-      goto skip_line;
-
-    lv_name = strtok_r (NULL, ":", &saveptr);
-    if (!lv_name)
-      goto skip_line;
-
-    /* Ignore thin layouts (RHBZ#1278878). */
-    if (lv_attr[0] == 't')
-      goto skip_line;
-
-    /* Ignore activationskip (RHBZ#1306666). */
-    if (strlen (lv_attr) >= 10 && lv_attr[9] == 'k')
-      goto skip_line;
-
-    /* Ignore "unknown device" message (RHBZ#1054761). */
-    if (STRNEQ (p, "unknown device")) {
-      char buf[256];
-
-      snprintf (buf, sizeof buf, "/dev/%s/%s", vg_name, lv_name);
-      if (add_string (&ret, buf) == -1) {
-        free (out);
-        return NULL;
-      }
-    }
-
-    p = pend;
-  }
-
-  free (out);
-
-  if (ret.size > 0)
-    sort_strings (ret.argv, ret.size);
-
-  if (end_stringsbuf (&ret) == -1)
-    return NULL;
-
-  return take_stringsbuf (&ret);
-}
-
 char **
 do_pvs (void)
 {
@@ -194,7 +111,7 @@ do_pvs (void)
   int r;
 
   r = command (&out, &err,
-               str_lvm, "pvs", "-o", "pv_name", "--noheadings", NULL);
+               "lvm", "pvs", "-o", "pv_name", "--noheadings", NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
     free (out);
@@ -212,7 +129,7 @@ do_vgs (void)
   int r;
 
   r = command (&out, &err,
-               str_lvm, "vgs", "-o", "vg_name", "--noheadings", NULL);
+               "lvm", "vgs", "-o", "vg_name", "--noheadings", NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
     free (out);
@@ -222,74 +139,6 @@ do_vgs (void)
   return convert_lvm_output (out, NULL);
 }
 
-/* Check whether lvs has -S to filter its output.
- * It is available only in lvm2 >= 2.02.107.
- */
-static int
-test_lvs_has_S_opt (void)
-{
-  static int result = -1;
-  if (result != -1)
-    return result;
-
-  CLEANUP_FREE char *out = NULL;
-  CLEANUP_FREE char *err = NULL;
-
-  int r = command (&out, &err, str_lvm, "lvs", "--help", NULL);
-  if (r == -1) {
-    reply_with_error ("lvm lvs --help: %s", err);
-    return -1;
-  }
-
-  if (strstr (out, "-S") == NULL)
-    result = 0;
-  else
-    result = 1;
-
-  return result;
-}
-
-char **
-do_lvs (void)
-{
-  char *out;
-  CLEANUP_FREE char *err = NULL;
-  int r;
-  const int has_S = test_lvs_has_S_opt ();
-
-  if (has_S < 0)
-    return NULL;
-
-  if (has_S > 0) {
-    r = command (&out, &err,
-                 str_lvm, "lvs",
-                 "-o", "vg_name,lv_name",
-                 "-S", "lv_role=public && lv_skip_activation!=1",
-                 "--noheadings",
-                 "--separator", "/", NULL);
-    if (r == -1) {
-      reply_with_error ("%s", err);
-      free (out);
-      return NULL;
-    }
-
-    return convert_lvm_output (out, "/dev/");
-  } else {
-    r = command (&out, &err,
-                 str_lvm, "lvs",
-                 "-o", "lv_attr,vg_name,lv_name",
-                 "--noheadings",
-                 "--separator", ":", NULL);
-    if (r == -1) {
-      reply_with_error ("%s", err);
-      free (out);
-      return NULL;
-    }
-
-    return filter_convert_old_lvs_output (out);
-  }
-}
-
 /* These were so complex to implement that I ended up auto-generating
  * the code.  That code is in stubs.c, and it is generated as usual
  * by generator.ml.
@@ -319,7 +168,7 @@ do_pvcreate (const char *device)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "pvcreate", "--force", device, NULL);
+               "lvm", "pvcreate", "--force", device, NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
     return -1;
@@ -337,13 +186,13 @@ do_vgcreate (const char *volgroup, char *const *physvols)
   CLEANUP_FREE char *err = NULL;
   CLEANUP_FREE const char **argv = NULL;
 
-  argc = count_strings (physvols) + 3;
+  argc = guestfs_int_count_strings (physvols) + 3;
   argv = malloc (sizeof (char *) * (argc + 1));
   if (argv == NULL) {
     reply_with_perror ("malloc");
     return -1;
   }
-  argv[0] = str_lvm;
+  argv[0] = "lvm";
   argv[1] = "vgcreate";
   argv[2] = volgroup;
   for (i = 3; i < argc+1; ++i)
@@ -370,7 +219,7 @@ do_lvcreate (const char *logvol, const char *volgroup, int mbytes)
   snprintf (size, sizeof size, "%d", mbytes);
 
   r = command (NULL, &err,
-               str_lvm, "lvcreate",
+               "lvm", "lvcreate",
                "-L", size, "-n", logvol, volgroup, NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
@@ -397,7 +246,7 @@ do_lvcreate_free (const char *logvol, const char *volgroup, int percent)
   snprintf (size, sizeof size, "%d%%FREE", percent);
 
   r = command (NULL, &err,
-               str_lvm, "lvcreate",
+               "lvm", "lvcreate",
                "-l", size, "-n", logvol, volgroup, NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
@@ -430,7 +279,7 @@ do_lvresize (const char *logvol, int mbytes)
   snprintf (size, sizeof size, "%d", mbytes);
 
   r = command (NULL, &err,
-               str_lvm, "lvresize",
+               "lvm", "lvresize",
                "--force", "-L", size, logvol, NULL);
   if (r == -1) {
     if (!ignore_same_size_error (err)) {
@@ -457,7 +306,7 @@ do_lvresize_free (const char *logvol, int percent)
   snprintf (size, sizeof size, "+%d%%FREE", percent);
 
   r = command (NULL, &err,
-               str_lvm, "lvresize", "-l", size, logvol, NULL);
+               "lvm", "lvresize", "-l", size, logvol, NULL);
   if (r == -1) {
     if (!ignore_same_size_error (err)) {
       reply_with_error ("%s", err);
@@ -489,10 +338,10 @@ do_lvm_remove_all (void)
       /* Deactivate the LV first.  On Ubuntu, lvremove '-f' option
        * does not remove active LVs reliably.
        */
-      (void) command (NULL, NULL, str_lvm, "lvchange", "-an", xs[i], NULL);
+      (void) command (NULL, NULL, "lvm", "lvchange", "-an", xs[i], NULL);
       udev_settle ();
 
-      r = command (NULL, &err, str_lvm, "lvremove", "-f", xs[i], NULL);
+      r = command (NULL, &err, "lvm", "lvremove", "-f", xs[i], NULL);
       if (r == -1) {
         reply_with_error ("lvremove: %s: %s", xs[i], err);
         return -1;
@@ -510,10 +359,10 @@ do_lvm_remove_all (void)
       CLEANUP_FREE char *err = NULL;
 
       /* Deactivate the VG first, see note above. */
-      (void) command (NULL, NULL, str_lvm, "vgchange", "-an", xs[i], NULL);
+      (void) command (NULL, NULL, "lvm", "vgchange", "-an", xs[i], NULL);
       udev_settle ();
 
-      r = command (NULL, &err, str_lvm, "vgremove", "-f", xs[i], NULL);
+      r = command (NULL, &err, "lvm", "vgremove", "-f", xs[i], NULL);
       if (r == -1) {
         reply_with_error ("vgremove: %s: %s", xs[i], err);
         return -1;
@@ -530,7 +379,7 @@ do_lvm_remove_all (void)
     for (i = 0; xs[i] != NULL; ++i) {
       CLEANUP_FREE char *err = NULL;
 
-      r = command (NULL, &err, str_lvm, "pvremove", "-f", xs[i], NULL);
+      r = command (NULL, &err, "lvm", "pvremove", "-f", xs[i], NULL);
       if (r == -1) {
         reply_with_error ("pvremove: %s: %s", xs[i], err);
         return -1;
@@ -551,7 +400,7 @@ do_lvremove (const char *device)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "lvremove", "-f", device, NULL);
+               "lvm", "lvremove", "-f", device, NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
     return -1;
@@ -569,7 +418,7 @@ do_vgremove (const char *device)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "vgremove", "-f", device, NULL);
+               "lvm", "vgremove", "-f", device, NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
     return -1;
@@ -587,7 +436,7 @@ do_pvremove (const char *device)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "pvremove", "-ff", device, NULL);
+               "lvm", "pvremove", "-ff", device, NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
     return -1;
@@ -605,7 +454,7 @@ do_pvresize (const char *device)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "pvresize", device, NULL);
+               "lvm", "pvresize", device, NULL);
   if (r == -1) {
     reply_with_error ("%s: %s", device, err);
     return -1;
@@ -624,7 +473,8 @@ do_pvresize_size (const char *device, int64_t size)
   snprintf (buf, sizeof buf, "%" PRIi64 "b", size);
 
   r = command (NULL, &err,
-               str_lvm, "pvresize",
+               "lvm", "pvresize",
+               "--yes",
                "--setphysicalvolumesize", buf,
                device, NULL);
   if (r == -1) {
@@ -642,14 +492,14 @@ do_vg_activate (int activate, char *const *volgroups)
   CLEANUP_FREE char *err = NULL;
   CLEANUP_FREE const char **argv = NULL;
 
-  argc = count_strings (volgroups) + 4;
+  argc = guestfs_int_count_strings (volgroups) + 4;
   argv = malloc (sizeof (char *) * (argc+1));
   if (argv == NULL) {
     reply_with_perror ("malloc");
     return -1;
   }
 
-  argv[0] = str_lvm;
+  argv[0] = "lvm";
   argv[1] = "vgchange";
   argv[2] = "-a";
   argv[3] = activate ? "y" : "n";
@@ -681,7 +531,7 @@ do_lvrename (const char *logvol, const char *newlogvol)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "lvrename",
+               "lvm", "lvrename",
                logvol, newlogvol, NULL);
   if (r == -1) {
     reply_with_error ("%s -> %s: %s", logvol, newlogvol, err);
@@ -700,7 +550,7 @@ do_vgrename (const char *volgroup, const char *newvolgroup)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "vgrename",
+               "lvm", "vgrename",
                volgroup, newvolgroup, NULL);
   if (r == -1) {
     reply_with_error ("%s -> %s: %s", volgroup, newvolgroup, err);
@@ -718,7 +568,7 @@ get_lvm_field (const char *cmd, const char *field, const char *device)
   char *out;
   CLEANUP_FREE char *err = NULL;
   int r = command (&out, &err,
-                   str_lvm, cmd,
+                   "lvm", cmd,
                    "--unbuffered", "--noheadings", "-o", field,
                    device, NULL);
   if (r == -1) {
@@ -755,7 +605,7 @@ get_lvm_fields (const char *cmd, const char *field, const char *device)
   CLEANUP_FREE char *out = NULL, *err = NULL;
 
   int r = command (&out, &err,
-                   str_lvm, cmd,
+                   "lvm", cmd,
                    "--unbuffered", "--noheadings", "-o", field,
                    device, NULL);
   if (r == -1) {
@@ -789,12 +639,28 @@ do_vglvuuids (const char *vgname)
 
 int
 do_vgscan (void)
+{
+  return do_lvm_scan (0);
+}
+
+int
+do_lvm_scan (int activate)
 {
   CLEANUP_FREE char *err = NULL;
   int r;
+  const char *argv[MAX_ARGS];
+  size_t i = 0;
 
-  r = command (NULL, &err,
-               str_lvm, "vgscan", NULL);
+  ADD_ARG (argv, i, "lvm");
+  ADD_ARG (argv, i, "pvscan");
+  ADD_ARG (argv, i, "--cache");
+  if (activate) {
+    ADD_ARG (argv, i, "--activate");
+    ADD_ARG (argv, i, "ay");
+  }
+  ADD_ARG (argv, i, NULL);
+
+  r = commandv (NULL, &err, (const char * const *) argv);
   if (r == -1) {
     reply_with_error ("%s", err);
     return -1;
@@ -875,7 +741,18 @@ char *
 do_lvm_canonical_lv_name (const char *device)
 {
   char *canonical;
-  int r = lv_canonical (device, &canonical);
+  int r;
+
+  /* The device parameter is passed as PlainString because we can't
+   * really be sure that the device name will exist (especially for
+   * "/dev/mapper/..." names).  Do some sanity checking on it here.
+   */
+  if (!STRPREFIX (device, "/dev/")) {
+    reply_with_error ("%s: not a device name", device);
+    return NULL;
+  }
+
+  r = lv_canonical (device, &canonical);
   if (r == -1)
     return NULL;
 
@@ -982,7 +859,7 @@ do_vgmeta (const char *vg, size_t *size_r)
 
   close (fd);
 
-  r = command (NULL, &err, str_lvm, "vgcfgbackup", "-f", tmp, vg, NULL);
+  r = command (NULL, &err, "lvm", "vgcfgbackup", "-f", tmp, vg, NULL);
   if (r == -1) {
     reply_with_error ("vgcfgbackup: %s", err);
     return NULL;
@@ -1055,7 +932,7 @@ do_pvchange_uuid (const char *device)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "pvchange", "-u", device, NULL);
+               "lvm", "pvchange", "-u", device, NULL);
   if (r == -1) {
     reply_with_error ("%s: %s", device, err);
     return -1;
@@ -1073,7 +950,7 @@ do_pvchange_uuid_all (void)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "pvchange", "-u", "-a", NULL);
+               "lvm", "pvchange", "-u", "-a", NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
     return -1;
@@ -1091,7 +968,7 @@ do_vgchange_uuid (const char *vg)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "vgchange", "-u", vg, NULL);
+               "lvm", "vgchange", "-u", vg, NULL);
   if (r == -1) {
     reply_with_error ("%s: %s", vg, err);
     return -1;
@@ -1109,7 +986,7 @@ do_vgchange_uuid_all (void)
   int r;
 
   r = command (NULL, &err,
-               str_lvm, "vgchange", "-u", NULL);
+               "lvm", "vgchange", "-u", NULL);
   if (r == -1) {
     reply_with_error ("%s", err);
     return -1;
