@@ -1,5 +1,5 @@
 (* libguestfs
- * Copyright (C) 2009-2019 Red Hat Inc.
+ * Copyright (C) 2009-2020 Red Hat Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,43 +42,28 @@ let rec generate_python_actions_h () =
 #define GUESTFS_PYTHON_ACTIONS_H_
 
 #include \"guestfs.h\"
-#include \"guestfs-utils.h\"
+#include \"guestfs-stringlists-utils.h\"
 
-#if PY_VERSION_HEX < 0x02050000
-typedef int Py_ssize_t;
-#define PY_SSIZE_T_MAX INT_MAX
-#define PY_SSIZE_T_MIN INT_MIN
-#endif
+#define PY_SSIZE_T_CLEAN 1
 
-#ifndef HAVE_PYCAPSULE_NEW
-typedef struct {
-  PyObject_HEAD
-  guestfs_h *g;
-} Pyguestfs_Object;
-#endif
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored \"-Wcast-align\"
+#include <Python.h>
+#pragma GCC diagnostic pop
 
 static inline guestfs_h *
 get_handle (PyObject *obj)
 {
   assert (obj);
   assert (obj != Py_None);
-#ifndef HAVE_PYCAPSULE_NEW
-  return ((Pyguestfs_Object *) obj)->g;
-#else
   return (guestfs_h*) PyCapsule_GetPointer(obj, \"guestfs_h\");
-#endif
 }
 
 static inline PyObject *
 put_handle (guestfs_h *g)
 {
   assert (g);
-#ifndef HAVE_PYCAPSULE_NEW
-  return
-    PyCObject_FromVoidPtrAndDesc ((void *) g, (char *) \"guestfs_h\", NULL);
-#else
   return PyCapsule_New ((void *) g, \"guestfs_h\", NULL);
-#endif
 }
 
 extern void guestfs_int_py_extend_module (PyObject *module);
@@ -134,12 +119,6 @@ and generate_python_structs () =
   generate_header CStyle LGPLv2plus;
 
   pr "\
-/* This has to be included first, else definitions conflict with
- * glibc header files.  Python is broken.
- */
-#define PY_SSIZE_T_CLEAN 1
-#include <Python.h>
-
 #include <config.h>
 
 #include <stdio.h>
@@ -199,7 +178,7 @@ and generate_python_structs () =
             pr "  value = PyBytes_FromStringAndSize (%s->%s, %s->%s_len);\n"
               typ name typ name;
             pr "#else\n";
-            pr "  value = guestfs_int_py_fromstringsize (%s->%s, %s->%s_len);\n"
+            pr "  value = PyString_FromStringAndSize (%s->%s, %s->%s_len);\n"
               typ name typ name;
             pr "#endif\n";
             pr "  if (value == NULL)\n";
@@ -272,14 +251,12 @@ and generate_python_actions actions () =
   generate_header CStyle LGPLv2plus;
 
   pr "\
-/* This has to be included first, else definitions conflict with
- * glibc header files.  Python is broken.
- */
-#define PY_SSIZE_T_CLEAN 1
-#include <Python.h>
-
 #include <config.h>
 
+/* It is safe to call deprecated functions from this file. */
+#define GUESTFS_NO_WARN_DEPRECATED
+#undef GUESTFS_NO_DEPRECATED
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
@@ -519,7 +496,7 @@ and generate_python_actions actions () =
            pr "#if PY_MAJOR_VERSION >= 3\n";
            pr "  py_r = PyBytes_FromStringAndSize (r, size);\n";
            pr "#else\n";
-           pr "  py_r = guestfs_int_py_fromstringsize (r, size);\n";
+           pr "  py_r = PyString_FromStringAndSize (r, size);\n";
            pr "#endif\n";
            pr "  free (r);\n";
            pr "  if (py_r == NULL) goto out;\n";
@@ -565,12 +542,6 @@ and generate_python_module () =
   generate_header CStyle LGPLv2plus;
 
   pr "\
-/* This has to be included first, else definitions conflict with
- * glibc header files.  Python is broken.
- */
-#define PY_SSIZE_T_CLEAN 1
-#include <Python.h>
-
 #include <config.h>
 
 #include <stdio.h>
@@ -821,7 +792,6 @@ class GuestFS(object):
         \"\"\"Delete an event callback.\"\"\"
         self._check_not_closed()
         libguestfsmod.delete_event_callback(self._o, event_handle)
-
 ";
 
   let map_join f l =
@@ -838,6 +808,7 @@ class GuestFS(object):
           args ^
         map_join (fun optarg -> sprintf ", %s=None" (name_of_optargt optarg))
           optargs in
+      pr "\n";
       pr "    def %s(%s):\n"
         f.name (indent_python decl_string (9 + len_name) 78);
 
@@ -906,6 +877,18 @@ class GuestFS(object):
           pr "        %s = %s.c_pointer()\n" n n
       ) args;
       pr "        self._check_not_closed()\n";
+      (match f.deprecated_by with
+      | Not_deprecated -> ()
+      | Replaced_by alt ->
+        pr "        import warnings\n";
+        pr "        warnings.warn(\"use GuestFS.%s() \"\n" alt;
+        pr "                      \"instead of GuestFS.%s()\",\n" f.name;
+        pr "                      DeprecationWarning, stacklevel=2)\n";
+      | Deprecated_no_replacement ->
+        pr "        import warnings\n";
+        pr "        warnings.warn(\"do not use GuestFS.%s()\",\n" f.name;
+        pr "                      DeprecationWarning, stacklevel=2)\n";
+      );
       let function_string =
         "self._o" ^
         map_join (fun arg -> sprintf ", %s" (name_of_argt arg))
@@ -923,12 +906,11 @@ class GuestFS(object):
       );
 
       pr "        return r\n";
-      pr "\n";
 
       (* Aliases. *)
       List.iter (
         fun alias ->
-          pr "    %s = %s\n\n" alias f.name
+          pr "\n    %s = %s\n" alias f.name
       ) f.non_c_aliases
   ) (actions |> external_functions |> sort)
 
