@@ -1,5 +1,5 @@
 /* libguestfs
- * Copyright (C) 2009-2019 Red Hat Inc.
+ * Copyright (C) 2009-2020 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -134,6 +134,16 @@ struct backend_libvirt_data {
   size_t nr_secrets;
   char *uefi_code;		/* UEFI (firmware) code and variables. */
   char *uefi_vars;
+  char *default_qemu;           /* default qemu (from domcapabilities) */
+  char **firmware_autoselect;   /* supported firmwares (from domcapabilities);
+                                 * NULL means "not supported", otherwise it
+                                 * contains a list with supported values for
+                                 * <os firmware='...'>
+                                 */
+  const char *firmware;         /* firmware to set in autoselection mode;
+                                 * points to one of the elements in
+                                 * "firmware_autoselect"
+                                 */
   char guestfsd_path[UNIX_PATH_MAX]; /* paths to sockets */
   char console_path[UNIX_PATH_MAX];
 };
@@ -143,8 +153,9 @@ struct backend_libvirt_data {
  */
 struct libvirt_xml_params {
   struct backend_libvirt_data *data;
-  char *kernel;                 /* paths to kernel and initrd */
+  char *kernel;                 /* paths to kernel, initrd and appliance */
   char *initrd;
+  char *appliance;
   char *appliance_overlay;      /* path to qcow2 overlay backed by appliance */
   char appliance_dev[64];       /* appliance device name */
   size_t appliance_index;       /* index of appliance */
@@ -153,6 +164,7 @@ struct libvirt_xml_params {
 };
 
 static int parse_capabilities (guestfs_h *g, const char *capabilities_xml, struct backend_libvirt_data *data);
+static int parse_domcapabilities (guestfs_h *g, const char *domcapabilities_xml, struct backend_libvirt_data *data);
 static int add_secret (guestfs_h *g, virConnectPtr conn, struct backend_libvirt_data *data, const struct drive *drv);
 static int find_secret (guestfs_h *g, const struct backend_libvirt_data *data, const struct drive *drv, const char **type, const char **uuid);
 static int have_secret (guestfs_h *g, const struct backend_libvirt_data *data, const struct drive *drv);
@@ -161,7 +173,7 @@ static void debug_appliance_permissions (guestfs_h *g);
 static void debug_socket_permissions (guestfs_h *g);
 static void libvirt_error (guestfs_h *g, const char *fs, ...) __attribute__((format (printf,2,3)));
 static void libvirt_debug (guestfs_h *g, const char *fs, ...) __attribute__((format (printf,2,3)));
-static int is_custom_hv (guestfs_h *g);
+static int is_custom_hv (guestfs_h *g, struct backend_libvirt_data *data);
 static int is_blk (const char *path);
 static void ignore_errors (void *ignore, virErrorPtr ignore2);
 static void set_socket_create_context (guestfs_h *g);
@@ -312,10 +324,10 @@ launch_libvirt (guestfs_h *g, void *datav, const char *libvirt_uri)
     .data = data,
     .kernel = NULL,
     .initrd = NULL,
+    .appliance = NULL,
     .appliance_overlay = NULL,
   };
   CLEANUP_FREE xmlChar *xml = NULL;
-  CLEANUP_FREE char *appliance = NULL;
   struct sockaddr_un addr;
   struct drive *drv;
   size_t i;
@@ -324,6 +336,7 @@ launch_libvirt (guestfs_h *g, void *datav, const char *libvirt_uri)
   CLEANUP_FREE void *buf = NULL;
   unsigned long version_number;
   int uefi_flags;
+  CLEANUP_FREE char *domcapabilities_xml = NULL;
 
   params.current_proc_is_root = geteuid () == 0;
 
@@ -419,8 +432,28 @@ launch_libvirt (guestfs_h *g, void *datav, const char *libvirt_uri)
   if (parse_capabilities (g, capabilities_xml, data) == -1)
     goto cleanup;
 
+  domcapabilities_xml = virConnectGetDomainCapabilities (conn, NULL, NULL,
+#ifdef MACHINE_TYPE
+                                                         MACHINE_TYPE,
+#else
+                                                         NULL,
+#endif
+                                                         NULL, 0);
+  if (!domcapabilities_xml) {
+    libvirt_error (g, _("could not get libvirt domain capabilities"));
+    goto cleanup;
+  }
+
+  /* Parse domcapabilities XML.
+   */
+  debug (g, "parsing domcapabilities XML");
+
+  if (parse_domcapabilities (g, domcapabilities_xml, data) == -1)
+    goto cleanup;
+
   /* UEFI code and variables, on architectures where that is required. */
-  if (guestfs_int_get_uefi (g, &data->uefi_code, &data->uefi_vars,
+  if (guestfs_int_get_uefi (g, data->firmware_autoselect, &data->firmware,
+                            &data->uefi_code, &data->uefi_vars,
                             &uefi_flags) == -1)
     goto cleanup;
   if (uefi_flags & UEFI_FLAG_SECURE_BOOT_REQUIRED) {
@@ -454,18 +487,18 @@ launch_libvirt (guestfs_h *g, void *datav, const char *libvirt_uri)
   debug (g, "build appliance");
 
   if (guestfs_int_build_appliance (g, &params.kernel, &params.initrd,
-                                   &appliance) == -1)
+                                   &params.appliance) == -1)
     goto cleanup;
 
   guestfs_int_launch_send_progress (g, 3);
   TRACE0 (launch_build_libvirt_appliance_end);
 
   /* Note that appliance can be NULL if using the old-style appliance. */
-  if (appliance) {
+  if (params.appliance) {
 #ifndef APPLIANCE_FORMAT_AUTO
-    params.appliance_overlay = make_qcow2_overlay (g, appliance, "raw");
+    params.appliance_overlay = make_qcow2_overlay (g, params.appliance, "raw");
 #else
-    params.appliance_overlay = make_qcow2_overlay (g, appliance, NULL);
+    params.appliance_overlay = make_qcow2_overlay (g, params.appliance, NULL);
 #endif
     if (!params.appliance_overlay)
       goto cleanup;
@@ -589,7 +622,7 @@ launch_libvirt (guestfs_h *g, void *datav, const char *libvirt_uri)
   params.appliance_index = g->nr_drives;
   strcpy (params.appliance_dev, "/dev/sd");
   guestfs_int_drive_name (params.appliance_index, &params.appliance_dev[7]);
-  params.enable_svirt = ! is_custom_hv (g);
+  params.enable_svirt = ! is_custom_hv (g, data);
 
   xml = construct_libvirt_xml (g, &params);
   if (!xml)
@@ -682,7 +715,7 @@ launch_libvirt (guestfs_h *g, void *datav, const char *libvirt_uri)
     goto cleanup;
   }
 
-  if (appliance)
+  if (params.appliance)
     guestfs_int_add_dummy_appliance_drive (g);
 
   TRACE0 (launch_libvirt_end);
@@ -694,6 +727,7 @@ launch_libvirt (guestfs_h *g, void *datav, const char *libvirt_uri)
 
   free (params.kernel);
   free (params.initrd);
+  free (params.appliance);
   free (params.appliance_overlay);
 
   return 0;
@@ -719,6 +753,7 @@ launch_libvirt (guestfs_h *g, void *datav, const char *libvirt_uri)
 
   free (params.kernel);
   free (params.initrd);
+  free (params.appliance);
   free (params.appliance_overlay);
 
   g->state = CONFIG;
@@ -820,13 +855,74 @@ parse_capabilities (guestfs_h *g, const char *capabilities_xml,
 }
 
 static int
-is_custom_hv (guestfs_h *g)
+parse_domcapabilities (guestfs_h *g, const char *domcapabilities_xml,
+                       struct backend_libvirt_data *data)
 {
+  CLEANUP_XMLFREEDOC xmlDocPtr doc = NULL;
+  CLEANUP_XMLXPATHFREECONTEXT xmlXPathContextPtr xpathCtx = NULL;
+  CLEANUP_XMLXPATHFREEOBJECT xmlXPathObjectPtr xpathObj = NULL;
+  xmlNodeSetPtr nodes;
+  size_t i;
+
+  doc = xmlReadMemory (domcapabilities_xml, strlen (domcapabilities_xml),
+                       NULL, NULL, XML_PARSE_NONET);
+  if (doc == NULL) {
+    error (g, _("unable to parse domain capabilities XML returned by libvirt"));
+    return -1;
+  }
+
+  xpathCtx = xmlXPathNewContext (doc);
+  if (xpathCtx == NULL) {
+    error (g, _("unable to create new XPath context"));
+    return -1;
+  }
+
+  /* This gives us the default QEMU. */
+#define XPATH_EXPR "string(/domainCapabilities/path/text())"
+  xpathObj = xmlXPathEvalExpression (BAD_CAST XPATH_EXPR, xpathCtx);
+  if (xpathObj == NULL) {
+    error (g, _("unable to evaluate XPath expression: %s"), XPATH_EXPR);
+    return -1;
+  }
+#undef XPATH_EXPR
+
+  assert (xpathObj->type == XPATH_STRING);
+  data->default_qemu = safe_strdup (g, (char *) xpathObj->stringval);
+
+  /*
+   * This gives us whether the firmware autoselection is supported,
+   * and which values are allowed.
+   */
+#define XPATH_EXPR "/domainCapabilities/os/enum[@name='firmware']/value"
+  xmlXPathFreeObject (xpathObj);
+  xpathObj = xmlXPathEvalExpression (BAD_CAST XPATH_EXPR, xpathCtx);
+  if (xpathObj == NULL) {
+    error (g, _("unable to evaluate XPath expression: %s"), XPATH_EXPR);
+    return -1;
+  }
+#undef XPATH_EXPR
+
+  assert (xpathObj->type == XPATH_NODESET);
+  nodes = xpathObj->nodesetval;
+  if (nodes != NULL) {
+    data->firmware_autoselect = safe_calloc (g, nodes->nodeNr + 1, sizeof (char *));
+    for (i = 0; i < (size_t) nodes->nodeNr; ++i)
+      data->firmware_autoselect[i] = (char *) xmlNodeGetContent(nodes->nodeTab[i]);
+  }
+
+  return 0;
+}
+
+static int
+is_custom_hv (guestfs_h *g, struct backend_libvirt_data *data)
+{
+  if (g->hv && STRNEQ (g->hv, data->default_qemu))
+    return 1;
 #ifdef QEMU
-  return g->hv && STRNEQ (g->hv, QEMU);
-#else
-  return 1;
+  if (STRNEQ (data->default_qemu, QEMU))
+    return 1;
 #endif
+  return 0;
 }
 
 #if HAVE_LIBSELINUX
@@ -951,6 +1047,7 @@ static int construct_libvirt_xml_disk (guestfs_h *g, const struct backend_libvir
 static int construct_libvirt_xml_disk_target (guestfs_h *g, xmlTextWriterPtr xo, size_t drv_index);
 static int construct_libvirt_xml_disk_driver_qemu (guestfs_h *g, const struct backend_libvirt_data *data, struct drive *drv, xmlTextWriterPtr xo, const char *format, const char *cachemode, enum discard discard, bool copyonread);
 static int construct_libvirt_xml_disk_address (guestfs_h *g, xmlTextWriterPtr xo, size_t drv_index);
+static int construct_libvirt_xml_disk_blockio (guestfs_h *g, xmlTextWriterPtr xo, int blocksize);
 static int construct_libvirt_xml_disk_source_hosts (guestfs_h *g, xmlTextWriterPtr xo, const struct drive_source *src);
 static int construct_libvirt_xml_disk_source_seclabel (guestfs_h *g, const struct backend_libvirt_data *data, xmlTextWriterPtr xo);
 static int construct_libvirt_xml_appliance (guestfs_h *g, const struct libvirt_xml_params *params, xmlTextWriterPtr xo);
@@ -1122,9 +1219,12 @@ construct_libvirt_xml_boot (guestfs_h *g,
   flags = 0;
   if (!params->data->is_kvm)
     flags |= APPLIANCE_COMMAND_LINE_IS_TCG;
-  cmdline = guestfs_int_appliance_command_line (g, params->appliance_dev, flags);
+  cmdline = guestfs_int_appliance_command_line (g, params->appliance, flags);
 
   start_element ("os") {
+    if (params->data->firmware)
+      attribute ("firmware", params->data->firmware);
+
     start_element ("type") {
 #ifdef MACHINE_TYPE
       attribute ("machine", MACHINE_TYPE);
@@ -1212,7 +1312,7 @@ construct_libvirt_xml_devices (guestfs_h *g,
     /* Path to hypervisor.  Only write this if the user has changed the
      * default, otherwise allow libvirt to choose the best one.
      */
-    if (is_custom_hv (g))
+    if (is_custom_hv (g, params->data))
       single_element ("emulator", g->hv);
 #if defined(__arm__)
     /* Hopefully temporary hack to make ARM work (otherwise libvirt
@@ -1483,6 +1583,9 @@ construct_libvirt_xml_disk (guestfs_h *g,
     if (construct_libvirt_xml_disk_address (g, xo, drv_index) == -1)
       return -1;
 
+    if (construct_libvirt_xml_disk_blockio (g, xo, drv->blocksize) == -1)
+      return -1;
+
   } end_element (); /* </disk> */
 
   return 0;
@@ -1590,6 +1693,20 @@ construct_libvirt_xml_disk_address (guestfs_h *g, xmlTextWriterPtr xo,
   return 0;
 }
 
+static int
+construct_libvirt_xml_disk_blockio (guestfs_h *g, xmlTextWriterPtr xo,
+                                    int blocksize)
+{
+  if (blocksize) {
+    start_element ("blockio") {
+        attribute_format ("physical_block_size", "%d", blocksize);
+        attribute_format ("logical_block_size", "%d", blocksize);
+    } end_element ();
+  }
+
+  return 0;
+}
+
 static int
 construct_libvirt_xml_disk_source_hosts (guestfs_h *g,
                                          xmlTextWriterPtr xo,
@@ -2014,6 +2131,12 @@ shutdown_libvirt (guestfs_h *g, void *datav, int check_for_errors)
   free (data->uefi_vars);
   data->uefi_vars = NULL;
 
+  free (data->default_qemu);
+  data->default_qemu = NULL;
+
+  guestfs_int_free_string_list (data->firmware_autoselect);
+  data->firmware_autoselect = NULL;
+
   return ret;
 }
 
