@@ -1,5 +1,5 @@
 /* libguestfs python bindings
- * Copyright (C) 2009-2019 Red Hat Inc.
+ * Copyright (C) 2009-2020 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,12 +22,6 @@
  * F<python/actions-*.c>).
  */
 
-/* This has to be included first, else definitions conflict with
- * glibc header files.  Python is broken.
- */
-#define PY_SSIZE_T_CLEAN 1
-#include <Python.h>
-
 #include <config.h>
 
 #include <stdio.h>
@@ -118,13 +112,22 @@ guestfs_int_py_event_callback_wrapper (guestfs_h *g,
                                    const char *buf, size_t buf_len,
                                    const uint64_t *array, size_t array_len)
 {
-  PyGILState_STATE py_save = PyGILState_UNLOCKED;
+  PyGILState_STATE py_save;
   PyObject *py_callback = callback;
   PyObject *py_array;
   PyObject *args;
   PyObject *a;
   size_t i;
   PyObject *py_r;
+  int threads_initialized = PyEval_ThreadsInitialized ();
+
+#ifdef HAVE_PY_ISFINALIZING
+  if (_Py_IsFinalizing ())
+    return;
+#endif
+
+  if (threads_initialized)
+    py_save = PyGILState_Ensure ();
 
   py_array = PyList_New (array_len);
   for (i = 0; i < array_len; ++i) {
@@ -138,14 +141,8 @@ guestfs_int_py_event_callback_wrapper (guestfs_h *g,
                         buf, buf_len, py_array);
   Py_INCREF (args);
 
-  if (PyEval_ThreadsInitialized ())
-    py_save = PyGILState_Ensure ();
-
   py_r = PyObject_CallObject (py_callback, args);
 
-  if (PyEval_ThreadsInitialized ())
-    PyGILState_Release (py_save);
-
   Py_DECREF (args);
 
   if (py_r != NULL)
@@ -153,6 +150,9 @@ guestfs_int_py_event_callback_wrapper (guestfs_h *g,
   else
     /* Callback threw an exception: print it. */
     PyErr_PrintEx (0);
+
+  if (threads_initialized)
+    PyGILState_Release (py_save);
 }
 
 PyObject *
@@ -384,7 +384,7 @@ guestfs_int_py_put_table (char * const * const argv)
 PyObject *
 guestfs_int_py_fromstring (const char *str)
 {
-#ifdef HAVE_PYSTRING_ASSTRING
+#if PY_MAJOR_VERSION < 3
   return PyString_FromString (str);
 #else
   return PyUnicode_FromString (str);
@@ -394,7 +394,7 @@ guestfs_int_py_fromstring (const char *str)
 PyObject *
 guestfs_int_py_fromstringsize (const char *str, size_t size)
 {
-#ifdef HAVE_PYSTRING_ASSTRING
+#if PY_MAJOR_VERSION < 3
   return PyString_FromStringAndSize (str, size);
 #else
   return PyUnicode_FromStringAndSize (str, size);
@@ -404,7 +404,7 @@ guestfs_int_py_fromstringsize (const char *str, size_t size)
 char *
 guestfs_int_py_asstring (PyObject *obj)
 {
-#ifdef HAVE_PYSTRING_ASSTRING
+#if PY_MAJOR_VERSION < 3
   return PyString_AsString (obj);
 #else
   PyObject *bytes = PyUnicode_AsUTF8String (obj);
