@@ -26,7 +26,7 @@
 #include <error.h>
 
 #include "guestfs.h"
-#include "guestfs-internal-frontend.h"
+#include "guestfs-utils.h"
 
 int
 main (int argc, char *argv[])
